clear;close all;

addpath(genpath('datagen'));
addpath(genpath('func'));
addpath(genpath('toolbox'));

%% generate adjacency matrix using SB model with 3 blocks
N = 100;
b1 = 0.2 * N;
b2 = 0.4 * N;
b3 = 0.4 * N;

den = 0.15;
sb1 = construct_graph(b1, 'er', den);
sb2 = construct_graph(b2, 'er', den);
sb3 = construct_graph(b3, 'er', den);

Ab = blkdiag(sb1,sb2,sb3);

A0 = construct_graph(N,'er',0.02);
A = A0 + Ab;
A = (A>0) * 1;

[X, U] = SynDataGenSparseU_sb3(A, 0.9, 500);

% add noisy to data
Xt = X;
Xt(Xt<10^-3) = 0;
mu = mean(Xt(Xt~=0),"all");
sd = std(Xt(Xt~=0));
nsr = 0.5;
X_noisy = X + nsr * sd .* randn(size(X));

% get vector form of GT adjacency matrix A
mask = triu(true([N N]),1);
Av = A(mask);
Av = Av(:);


%% GIST(S) Spline Dictionary
param.alpha = 1;
param.beta = 10;
param.gama = 1;
param.N = N;
param.max_iter = 200;

[L_gist_spline, ~] = graph_learning_gist(X_noisy, param, 'fast', 'spline');
A_gist_spline = diag(diag(L_gist_spline)) - L_gist_spline;

% get vector form and calc AUC
Ah = A_gist_spline(mask);
Ah = Ah(:);
[~,~,~, auc_s] = perfcurve(Av, Ah,'1');


%% GIST(R) Ramanujuan Dictionary
param.alpha = 0.00001;
param.beta = 0.1;
param.gama = 10;
param.maxperiod = 10;
param.N = N;
param.max_iter = 200;

[L_gist_rama, ~] = graph_learning_gist(X_noisy, param, 'fast', 'rama');
A_gist_rama = diag(diag(L_gist_rama)) - L_gist_rama;

% get vector form and calc AUC
Ah = A_gist_rama(mask);
Ah = Ah(:);
[~,~,~, auc_r] = perfcurve(Av, Ah,'1');


%%  GIST-SP Ramanujuan Dictionary
param.alpha = 0.001;
param.beta = 100000;
param.gama = 100;
param.maxperiod = 10;
param.lr = 0.0001;
param.gd_iter = 200;
param.N = N;
param.max_iter = 200;

[L_gist_sp, ~] = graph_learning_gist_sp(X_noisy, param, 'fast');
A_gist_sp = diag(diag(L_gist_sp)) - L_gist_sp;

% get vector form and calc AUC
Ah = A_gist_sp(mask);
Ah = Ah(:);
[~,~,~, auc_sp] = perfcurve(Av, Ah,'1');


%% plot result
%  Since the synthetic data is genrated by Ramanujuan Dictionary, GIST(R)
%  performs the best
bar([auc_s, auc_r, auc_sp])
xticklabels(["GIST(S)", "GIST(R)", "GIST-SP"]);
ylabel("AUC");
