function [L, U] = graph_learning_gist(X_noisy, param, option, dic)

N = param.N;
max_iter = param.max_iter;
alpha = param.alpha;
beta = param.beta;
gama = param.gama;
e = 1e-4;

[n,t]=size(X_noisy);

switch dic
    case 'spline'
        phi = construct_spline_dictionary([1:t]);
        phi = phi';
    case 'rama'
        max_period=param.maxperiod;
        phi = Create_Dictionary(max_period,t,'Ramanujan');
        phi = phi';
    case 'dft'
        phi=(1/(t^(1/2)))*dftmtx(t);
end

U = rand(n, size(phi, 1));

%% new variabls

XPhiT = X_noisy * phi';
V = U;
UP = U;
LP = rand(N);

pa.maxit = 100;
Gamma = 0;
rho = gama/10;

[pr, pc] = size(phi);

if string(dic) == "rama" && param.maxperiod > 80 && ((pr/pc > 2) || (pr/pc < 0.5))
    [QB, S, ~] = svd(phi);
    dB = diag(2 * (S*S') + rho*eye(size(S,1)));
else
    phiPhiT=phi*phi';
    phiEye = eye(size(phiPhiT));
    B = 2*phiPhiT + rho*phiEye;
    [QB, dB] = eig(B, 'vector');
end

for i = 1:max_iter

    % Step 1: given Y, update L
    switch option
        case 'dong'
            L = optimize_laplacian_gaussian(N,U,alpha,beta);

            %% glbox
        case 'fast'
            Z = gsp_distanz(U').^2;
            [W, c, ~] = gsp_learn_graph_l2_degrees_one_loop(alpha/2*Z, 2*beta, pa);
            W = max(0, W);
            W = N/sum(W,'all') * W;
            L = diag(sum(W))-W;
    end
    Ln = norm(L, 'fro');

    % Step 2: Given L, update U


    % Update V
    h = U - Gamma/rho;
    V = sign(h).*max(abs(h)-gama/rho,0);

    % Update Gamma
    Gamma = Gamma + rho*(V-U);

    % update U
    A = 2 * alpha * L;
    C = 2*XPhiT + rho*V + Gamma;
    U = pre_compute_sylvester(A, QB, dB, C);

    Un = norm(U,'fro');

    UNP = norm(UP, 'fro');
    LNP = norm(LP, 'fro');


    if i>=2 && abs((Un - UNP)/UNP) < e && abs((Ln - LNP)/LNP) < e && abs((c-cp)/cp) < e
        fprintf('GIST CONVERGED at # iters: %4d. U_norm: %6.4f; L_norm: %6.4f; c: %6.4f\n', i, Un, Ln, c);
        break;
    end
    fprintf('GIST # iters: %4d. normX: %6.4f; trace: %6.4f; normU: %6.4f; normL: %6.4f\n', i, norm(X_noisy-U*phi, 'fro'), trace(U'*L*U), Un, Ln);

    UP = U;
    LP = L;
    cp = c;
    meta = i;

end