function [L, U] = graph_learning_gist_sp(X_noisy, param, option)

N = param.N;
max_iter = param.max_iter;
alpha = param.alpha;
beta = param.beta;
gama = param.gama;
lr = param.lr;
gd_iter = param.gd_iter;
e = 1e-4;

[n,t]=size(X_noisy);
max_period=param.maxperiod;
phi = Create_Dictionary(max_period,t,'Ramanujan');
phi = phi';
[k, ~] = size(phi);
[periods,~]  =  get_period_penalty(max_period);
P = get_ram_dicionary_indicator_mtrx(periods); % capture block structure
P = sparse(P);
U = rand(n, size(phi, 1));

V = U;
U_pre = U;
L_pre = rand(N);

pa.maxit = 100;
Gamma = zeros(n,k);
rho = gama/10;
phiPhiT = phi * phi';
xPhiT = X_noisy * phi';
ppT = P * P';

for i = 1:max_iter

    % Step 1: given Y, update L
    U_sp = abs(U);
    switch option
        case 'dong'
            L = optimize_laplacian_gaussian(N,U_sp,alpha,beta);
            %% glbox
        case 'fast'
            Z = gsp_distanz(U_sp').^2;
            [W, c, ~] = gsp_learn_graph_l2_degrees_one_loop(alpha/2*Z, 2*beta, pa); % Dong's
            W = max(0, W);
            W = N/sum(W,'all') * W;
            L = diag(sum(W))-W;
    end

    %% Step 2: Given L, update U
    Us = abs(U)*P;
    obj_pre = norm(X_noisy-U*phi,'fro') + alpha*trace(Us'*L*Us) + gama*sum(abs(V),"all") + rho/2 * norm(V-U+Gamma/rho, "fro");
    for j=1:gd_iter
        Un = U - lr * (2*U*phiPhiT - 2*xPhiT + (2*alpha*L*abs(U)*ppT).*sign(U) + rho*U -rho*V - Gamma);
        Uns = abs(Un)*P;
        obj = norm(X_noisy-Un*phi,'fro') + alpha*trace(Uns'*L*Uns) + gama*sum(abs(V),"all") + rho/2 * norm(V-Un+Gamma/rho, "fro");

        if isnan(norm(Un, 'fro')) || isinf(norm(Un, 'fro')) || isnan(obj) || isinf(obj)
            % disp("nan/inf break at iter: " + string(j))
            break;
        end

        if abs((obj - obj_pre)/obj_pre) < e || obj > obj_pre
            % disp("GD converged at iter: " + string(j))
            break;
        end

        U = Un;
        obj_pre = obj;
    end

    % Update V
    h = U - Gamma/rho;
    V = sign(h).*max(abs(h)-gama/rho,0);

    % Update Gamma
    Gamma = Gamma + rho*(V-U);

    U_pre_norm = norm(U_pre, 'fro');
    L_pre_norm = norm(L_pre, 'fro');
    U_norm = norm(U, 'fro');
    L_norm = norm(L, 'fro');

    if i>=2 && abs((U_norm - U_pre_norm)/U_pre_norm) < e && abs((L_norm - L_pre_norm)/L_pre_norm) < e
        fprintf('GIST-SP CONVERGED # iters: %4d.', i);
        break;
    end
    fprintf('GIST-SP # iters: %4d.', i);

    U_pre = U;
    L_pre = L;

end
end

